<?php
namespace Zeedhi\DevOps\Model\Configurationmanager\Repositories;

use Doctrine\DBAL\Connection;
use Zeedhi\DevOps\Model\Configurationmanager\Entities\Subrelease as SubreleaseEntity;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;


class Branch extends \Doctrine\ORM\EntityRepository {

    public function findBranchesWithTaskInfoBySubRelease($subreleaseId, $organizationId, $filter = null) {
        $query = <<<SQL
        SELECT * 
          FROM (SELECT BR.ID, BR.NAME, RP.URL, BR.IS_ACTIVE,
                       LTRIM(REPLACE(CF.CHANGED_FILE, '\', '/'), '/') AS CHANGED_FILE
                  FROM BRANCH BR
                  JOIN SUBRELEASE SR ON SR.ID = BR.SUBRELEASE_ID
                  JOIN RELEASE RE ON RE.ID = SR.RELEASE_ID
             LEFT JOIN TASK_BRANCHES TB ON TB.NAME = BR.NAME
             LEFT JOIN CHANGED_FILES CF ON CF.TASK = TB.TASK
             LEFT JOIN REPOSITORY RP ON RP.ID = CF.REPOSITORY
                 WHERE BR.SUBRELEASE_ID = :SUBRELEASE
                   AND INSTR(BR.NAME, 'TASK_BKP') = 0
                   AND (RP.URL IS NULL OR 
                        RP.URL IN (SELECT URL
                                    FROM REPOSITORY
                                   WHERE PRODUCT = RE.PRODUCT_ID
                                     AND STATUS = 'A')))
         WHERE 1 = 1
              $filter
      ORDER BY ID, URL, CHANGED_FILE
SQL;
        $params = array(
            'SUBRELEASE' => $subreleaseId,
            'ORGANIZATION' => $organizationId,
        );
        return $this->getEntityManager()->getConnection()->fetchAll($query, $params);
    }

    /**
     * @param $branchName
     * @param $productId
     * @param $isPmUtilization
     * @param $organizationId
     * @return array
     */
    public function findBranchByNameAndProduct($branchNameFilter, $productId, $isPmUtilization, $organizationId, $subreleaseId) {
        $filterPm = '';
        if($isPmUtilization) {
            $filterPm = ' AND BR.NAME = (SELECT T.NAME
                                           FROM TASK_BRANCHES T
                                          WHERE T.NAME = BR.NAME)';
        }
        $query = <<<SQL
			SELECT PR.NAME AS PRODUCT_NAME, SR.STATUS, SR.ID AS SR_ID, BR.*,
			       CASE WHEN BR.SUBRELEASE_ID = :SUBRELEASE_ID
			            THEN 'INVISIBLE_BRANCH'
			            ELSE 'VISIBLE_BRANCH'
			        END AS VALIDATE_BRANCH
			  FROM BRANCH BR
		INNER JOIN PRODUCT PR ON BR.PRODUCT_ID  = PR.ID
		 LEFT JOIN SUBRELEASE SR ON BR.SUBRELEASE_ID = SR.ID
		     WHERE BR.PRODUCT_ID = :PRODUCT_ID
		       $branchNameFilter
		       AND PR.ORGANIZATION_ID = :ORGANIZATION
		       AND (NOT EXISTS (SELECT B.ID
		                          FROM BRANCH B
		                         WHERE B.NAME = BR.NAME
		                           AND B.SUBRELEASE_ID = :SUBRELEASE_ID)
		            OR BR.SUBRELEASE_ID = :SUBRELEASE_ID)
		       $filterPm
SQL;
        $params = array(
            'ORGANIZATION' => $organizationId,
            'PRODUCT_ID' => $productId,
            'SUBRELEASE_ID' => $subreleaseId
        );

        $branches = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return $branches;
    }

    /**
     * @param $organizationId
     * @param $subreleaseId
     * @return array
     */
    public function findBranchWithRepositoryUrlToExcludeBySubrelease($subreleaseId, $organizationId) {
        $query = <<<SQL
			SELECT BR.ID, BR.NAME,
                   RTRIM(XMLAGG(XMLELEMENT(E, BR.URL, ',').EXTRACT('//text()') ORDER BY BR.URL).GETCLOBVAL(), ',') AS URL
              FROM (SELECT BR.ID, BR.NAME, RE.URL
                      FROM BRANCH BR
                     INNER JOIN REPOSITORY_BRANCH RB ON RB.BRANCH_ID = BR.ID
                     INNER JOIN REPOSITORY RE ON RE.ID = RB.REPOSITORY_ID
                     WHERE SUBRELEASE_ID = :SUBRELEASE
                       AND RE.URL NOT IN (SELECT REP.URL
                                            FROM REPOSITORY REP
                                           INNER JOIN REPOSITORY_BRANCH RBR ON RBR.REPOSITORY_ID = REP.ID
                                           INNER JOIN BRANCH BRA ON BRA.ID = RBR.BRANCH_ID
                                           INNER JOIN SUBRELEASE SBR ON SBR.ID = BRA.SUBRELEASE_ID AND SBR.ID <> :SUBRELEASE
                                           WHERE BRA.NAME = BR.NAME
                                             AND SBR.STATUS IN (:SUBRELEASE_STATUS)
                                             AND SBR.IS_ACTIVE = :IS_ACTIVE
                                             AND SBR.ORGANIZATION_ID = :ORGANIZATION)) BR
             GROUP BY BR.ID, BR.NAME
SQL;
        $params = array(
            'ORGANIZATION' => $organizationId,
            'SUBRELEASE' => $subreleaseId,
            'SUBRELEASE_STATUS' => array(SubreleaseEntity::IN_CONFLICT_STATUS,
                                         SubreleaseEntity::PENDING_STATUS,
                                         SubreleaseEntity::IN_TEST_STATUS),
            'IS_ACTIVE' => DevOpsUtilities::IS_ACTIVE_YES
        );

        $paramTypes = array(
            'SUBRELEASE_STATUS' => Connection::PARAM_STR_ARRAY,
        );

        $branches = $this->getEntityManager()->getConnection()->fetchAll($query, $params, $paramTypes);
        return $branches;
    }

    public function findBranchWithRepositoryByProject($projectId, $subReleaseId = null) {
        $subReleaseFilter = $subReleaseId ? " AND SR.ID = $subReleaseId " : "";
        $query = <<<SQL
	  SELECT SR.ORGANIZATION_ID, BR.ID AS BRANCH_ID, BR.IS_ACTIVE, BR.NAME AS BRANCH_NAME, SR.ID AS SUBRELEASE_ID,
             SR.NAME AS SUBRELEASE_NAME, SR.BRANCH_TO_DEPLOY, SR.RELEASE_ID, RB.ID AS REPOSITORY_BRANCH_ID,
             RB.REPOSITORY_ID, RP.IS_MASTER, RP.SEQNO, RP.URL, RP.PATH, PR.ID AS PRODUCT_ID, PR.BASE_URL,
             RP.NAME AS REPOSITORY_NAME
        FROM SUBRELEASE SR
        JOIN BRANCH BR ON BR.SUBRELEASE_ID = SR.ID
        JOIN REPOSITORY_BRANCH RB ON RB.BRANCH_ID = BR.ID
        JOIN REPOSITORY RP ON RP.ID = RB.REPOSITORY_ID
        JOIN PRODUCT PR ON PR.ID         = BR.PRODUCT_ID
       WHERE RP.STATUS <> 'D'
         AND SR.PROJECT_ID = :PROJECT_ID
             $subReleaseFilter
    ORDER BY RP.URL,
             RP.IS_MASTER DESC,
             RP.SEQNO ASC,
             BR.ID DESC
SQL;
        $params = array(
            'PROJECT_ID' => $projectId,
        );
        $branches = $this->getEntityManager()->getConnection()->fetchAll($query, $params);
        return $branches;
    }
}